# Documentation

> This is the documentation's development version. The functionality described here may not be included in the current release version. Unreleased functionality can change or may not be included the next release. The current release version's documentation is available on the [TestCafe website](https://devexpress.github.io/testcafe/documentation/getting-started/).

* [GETTING STARTED](articles/documentation/getting-started/README.md)
* [USING TESTCAFE](articles/documentation/using-testcafe/README.md)
  * [Installing TestCafe](articles/documentation/using-testcafe/installing-testcafe.md)
  * [Command Line Interface](articles/documentation/using-testcafe/command-line-interface.md)
  * [Programming Interface](articles/documentation/using-testcafe/programming-interface/README.md)
    * [TestCafe Class](articles/documentation/using-testcafe/programming-interface/testcafe.md)
    * [BrowserConnection Class](articles/documentation/using-testcafe/programming-interface/browserconnection.md)
    * [Runner Class](articles/documentation/using-testcafe/programming-interface/runner.md)
    * [createTestCafe Factory](articles/documentation/using-testcafe/programming-interface/createtestcafe.md)
  * [Using TestCafe Docker Image](articles/documentation/using-testcafe/using-testcafe-docker-image.md)
  * [Common Concepts](articles/documentation/using-testcafe/common-concepts/README.md)
    * [Browsers](articles/documentation/using-testcafe/common-concepts/browsers/README.md)
      * [Browser Support](articles/documentation/using-testcafe/common-concepts/browsers/browser-support.md)
      * [Testing in Headless Mode](articles/documentation/using-testcafe/common-concepts/browsers/testing-in-headless-mode.md)
      * [Using Chrome Device Emulation](articles/documentation/using-testcafe/common-concepts/browsers/using-chrome-device-emulation.md)
      * [User Profiles](articles/documentation/using-testcafe/common-concepts/browsers/user-profiles.md)
    * [Concurrent Test Execution](articles/documentation/using-testcafe/common-concepts/concurrent-test-execution.md)
    * [Reporters](articles/documentation/using-testcafe/common-concepts/reporters.md)
    * [Connect to the TestCafe Server over HTTPS](articles/documentation/using-testcafe/common-concepts/connect-to-the-testcafe-server-over-https.md)
* [TEST API](articles/documentation/test-api/README.md)
  * [Test Code Structure](articles/documentation/test-api/test-code-structure.md)
    * [Fixtures](articles/documentation/test-api/test-code-structure.md#fixtures)
    * [Tests](articles/documentation/test-api/test-code-structure.md#tests)
      * [Test Controller](articles/documentation/test-api/test-code-structure.md#test-controller)
      * [Setting Test Speed](articles/documentation/test-api/test-code-structure.md#setting-test-speed)
    * [Specifying the Start Webpage](articles/documentation/test-api/test-code-structure.md#specifying-the-start-webpage)
    * [Specifying Testing Metadata](articles/documentation/test-api/test-code-structure.md#specifying-testing-metadata)
      * [Using Metadata in Reports](articles/documentation/test-api/test-code-structure.md#using-metadata-in-reports)
    * [Initialization and Clean-Up](articles/documentation/test-api/test-code-structure.md#initialization-and-clean-up)
      * [Test Hooks](articles/documentation/test-api/test-code-structure.md#test-hooks)
      * [Fixture Hooks](articles/documentation/test-api/test-code-structure.md#fixture-hooks)
    * [Skipping Tests](articles/documentation/test-api/test-code-structure.md#skipping-tests)
  * [Selecting Page Elements](articles/documentation/test-api/selecting-page-elements/README.md)
    * [Selectors](articles/documentation/test-api/selecting-page-elements/selectors/README.md)
      * [Creating Selectors](articles/documentation/test-api/selecting-page-elements/selectors/creating-selectors.md)
      * [Using Selectors](articles/documentation/test-api/selecting-page-elements/selectors/using-selectors.md)
      * [Functional-Style Selectors](articles/documentation/test-api/selecting-page-elements/selectors/functional-style-selectors.md)
      * [Selector Options](articles/documentation/test-api/selecting-page-elements/selectors/selector-options.md)
      * [Extending Selectors](articles/documentation/test-api/selecting-page-elements/selectors/extending-selectors.md)
      * [Edge Cases and Limitations](articles/documentation/test-api/selecting-page-elements/selectors/edge-cases-and-limitations.md)
    * [DOM Node State](articles/documentation/test-api/selecting-page-elements/dom-node-state.md)
    * [Framework-Specific Selectors](articles/documentation/test-api/selecting-page-elements/framework-specific-selectors.md)
    * [Examples of Working with DOM Elements](articles/documentation/test-api/selecting-page-elements/examples-of-working-with-dom-elements.md)
  * [Actions](articles/documentation/test-api/actions/README.md)
    * [Click](articles/documentation/test-api/actions/click.md)
    * [Double Click](articles/documentation/test-api/actions/double-click.md)
    * [Right Click](articles/documentation/test-api/actions/right-click.md)
    * [Drag Element](articles/documentation/test-api/actions/drag-element.md)
    * [Hover](articles/documentation/test-api/actions/hover.md)
    * [Select Text](articles/documentation/test-api/actions/select-text.md)
    * [Type Text](articles/documentation/test-api/actions/type-text.md)
    * [Press Key](articles/documentation/test-api/actions/press-key.md)
    * [Navigate](articles/documentation/test-api/actions/navigate.md)
    * [Take Screenshot](articles/documentation/test-api/actions/take-screenshot.md)
    * [Upload](articles/documentation/test-api/actions/upload.md)
    * [Resize Window](articles/documentation/test-api/actions/resize-window.md)
    * [Action Options](articles/documentation/test-api/actions/action-options.md)
  * [Assertions](articles/documentation/test-api/assertions/README.md)
    * [Assertion API](articles/documentation/test-api/assertions/assertion-api.md)
  * [Obtaining Data from the Client](articles/documentation/test-api/obtaining-data-from-the-client/README.md)
    * [Examples of Using Client Functions](articles/documentation/test-api/obtaining-data-from-the-client/examples-of-using-client-functions.md)
  * [Intercepting HTTP Requests](articles/documentation/test-api/intercepting-http-requests/README.md)
    * [Logging HTTP Requests](articles/documentation/test-api/intercepting-http-requests/logging-http-requests.md)
    * [Mocking HTTP Requests](articles/documentation/test-api/intercepting-http-requests/mocking-http-requests.md)
    * [Creating a Custom HTTP Request Hook](articles/documentation/test-api/intercepting-http-requests/creating-a-custom-http-request-hook.md)
    * [Specifying Which Requests are Handled by the Hook](articles/documentation/test-api/intercepting-http-requests/specifying-which-requests-are-handled-by-the-hook.md)
    * [Attaching Hooks to Tests and Fixtures](articles/documentation/test-api/intercepting-http-requests/attaching-hooks-to-tests-and-fixtures.md)
    * [requestOptions Object](articles/documentation/test-api/intercepting-http-requests/requestoptions-object.md)
  * [Built-In Waiting Mechanisms](articles/documentation/test-api/built-in-waiting-mechanisms.md)
  * [Authentication](articles/documentation/test-api/authentication/README.md)
    * [User Roles](articles/documentation/test-api/authentication/user-roles.md)
    * [HTTP Authentication](articles/documentation/test-api/authentication/http-authentication.md)
  * [Pausing the Test](articles/documentation/test-api/pausing-the-test.md)
  * [Handling Native Dialogs](articles/documentation/test-api/handling-native-dialogs.md)
  * [Working with &lt;iframes&gt;](articles/documentation/test-api/working-with-iframes.md)
  * [Debugging](articles/documentation/test-api/debugging.md)
  * [Accessing Console Messages](articles/documentation/test-api/accessing-console-messages.md)
  * [TypeScript Support](articles/documentation/test-api/typescript-support.md)
  * [CoffeeScript Support](articles/documentation/test-api/coffeescript-support.md)
  * [A-Z Index](articles/documentation/test-api/a-z-index.md)
* [EXTENDING TESTCAFE](articles/documentation/extending-testcafe/README.md)
  * [Reporter Plugin](articles/documentation/extending-testcafe/reporter-plugin/README.md)
    * [Reporter Methods](articles/documentation/extending-testcafe/reporter-plugin/reporter-methods.md)
    * [Helpers](articles/documentation/extending-testcafe/reporter-plugin/helpers.md)
  * [Browser Provider Plugin](articles/documentation/extending-testcafe/browser-provider-plugin/README.md)
    * [Browser Provider Methods](articles/documentation/extending-testcafe/browser-provider-plugin/browser-provider-methods.md)
* [CONTINUOUS INTEGRATION](articles/documentation/continuous-integration/README.md)
  * [AppVeyor](articles/documentation/continuous-integration/appveyor.md)
  * [Azure DevOps](articles/documentation/continuous-integration/azure-devops.md)
  * [CircleCI](articles/documentation/continuous-integration/circleci.md)
  * [GitLab](articles/documentation/continuous-integration/gitlab.md)
  * [Jenkins](articles/documentation/continuous-integration/jenkins.md)
  * [TeamCity](articles/documentation/continuous-integration/teamcity.md)
  * [Travis](articles/documentation/continuous-integration/travis.md)
  * [Travis + Sauce Labs](articles/documentation/continuous-integration/travis-and-sauce-labs.md)
* [RECIPES](articles/documentation/recipes/README.md)
  * [Debug in Chrome Developer Tools](articles/documentation/recipes/debug-in-chrome-dev-tools.md)
  * [Debug in Visual Studio Code](articles/documentation/recipes/debug-in-visual-studio-code.md)
  * [Debug in WebStorm](articles/documentation/recipes/debug-in-webstorm.md)
  * [Finding Code Issues with Flow Type Checker](articles/documentation/recipes/finding-code-issues-with-flow-type-checker.md)
  * [Running Tests Using Travis CI and Sauce Labs](articles/documentation/recipes/running-tests-using-travis-ci-and-sauce-labs.md)
  * [Testing &lt;select&gt; Elements](articles/documentation/recipes/testing-select-elements.md)
  * [Test Static HTML Pages](articles/documentation/recipes/test-static-html-pages.md)
  * [Testing on Remote Computers and Mobile Devices](articles/documentation/recipes/testing-on-remote-computers-and-mobile-devices.md)
  * [Using Page Model](articles/documentation/recipes/using-page-model.md)
  * [Using TestCafe with Grunt](articles/documentation/recipes/using-testcafe-with-grunt.md)
  * [Using TestCafe with Gulp](articles/documentation/recipes/using-testcafe-with-gulp.md)
